;##############################################################################;
;ROM template
use16
org 0
;##############################################################################;
macro printx x{
 local strng,next
 mov  si,strng
 call print
 jmp next
 strng db x,0
 next:
}
;##############################################################################;
macro printxln x{
 local strng,next
 mov  si,strng
 call print
 jmp next
 strng db x,0
 next:
 mov  si,newline
 call print
}
;##############################################################################;
ROM_SIZE=1          ;ROM size in 2Kb blocks
dw 0AA55h           ;BIOS signature
db ROM_SIZE*4       ;Size in 512 blocks
;##############################################################################;
start:
push cs
pop  ds

printxln "===FDD test ROM==="

mov  si,dsmsg
mov  ax,ds
call print_str_num

mov  si,esmsg
mov  ax,es
call print_str_num

mov  si,ssmsg
mov  ax,ss
call print_str_num

mov  si,spmsg
mov  ax,sp
call print_str_num

printxln ""
printxln "FDD test:"
call floppy_test

printxln ""
printxln "FDD load test:"
;call load_disk
;call check_crc
mov  ah,0
mov  si,crcmsg
call print_str_num

printxln "All Done. Press any key..."
;call getch

exit:
retf
;##############################################################################;
floppy_test:
mov  ax,cs
mov  ds,ax
mov  ax,1000h
mov  es,ax

;FDD reset
mov  ah,0
int  13h

mov  dl,0
mov  si,disk_a_msg
call one_disk
mov  dl,1
mov  si,disk_b_msg
call one_disk
mov  dl,2
mov  si,disk_c_msg
call one_disk
mov  dl,3
mov  si,disk_d_msg
call one_disk
ret

one_disk:
push si
xor  di,di
mov  cl,dl
mov  dx,0
call read_sect
jb   od_error
mov  ax,word[es:0]
pop  si
call print_str_num
jmp  od_fin

od_error:
pop  si
push ax
call print
pop  ax
mov  si,err_msg
call print_str_num

od_fin:
ret
;##############################################################################;
load_disk:
mov  cx,720-1
mov  ax,1000h
mov  es,ax

ldl:
xor  di,di
push cx
mov  dx,720-1
sub  dx,cx
mov  cl,0
call read_sect
pop  cx
jb   ld_error

mov  ax,es
add  ax,20h
mov  es,ax
loop ldl
jmp ld_fin

ld_error:
push ax
mov  ax,cx
mov  si,pos_msg
call print_str_num
pop  ax
mov  si,err_msg
call print_str_num
jmp ld_fin

ld_fin:
ret
;##############################################################################;
;Readsec Procedure. Reads absolute sector [dx] of [cl] to [es:di].
;Input:
;DX - Sector Number
;CL - Drive number
;ES:DI - Loading Pointer
read_sect:
push es cx di es

mov  cx,[TrkSecs]
mov  si,dx

mov  ax,si
xor  dx,dx
div  cx
mov  di,ax

mov  ax,di
imul cx
mov  dx,si
sub  dx,ax
inc  dx

mov  ax,di
and  ax,1
push ax   ;AbsHeadNum
push dx   ;AbsSectNum

imul cx
push ax
mov  ax,si
pop  dx
sub  ax,dx
pop  dx   ;AbsSectNum
push dx   ;AbsSectNum
dec  dx
sub  ax,dx
mov  dx,cx
shl  dx,1
push ax
push dx
xor  dx,dx
pop  bx
pop  ax
div  bx   

mov  cx,ax
mov  al,cl
shr  cx,2
and  cl,0C0h
mov  ch,al
and  cx,0FFC0h
pop  ax     ;AbsSectNum 
or   cl,al 
pop  ax     ;AbsHeadNum
pop  es bx 

pop  dx    
mov  dh,al  
mov  al,1
mov  ah,2
int  13h  ;DISK - READ SECTORS INTO MEMORY
          ;AH = 2, AL = number of sectors to read, CH = track, CL = sector
          ;DH = head, DL = drive, ES:BX -> buffer to fill
          ;Return: CF set on error, AH = status, AL = number of sectors read
pop  es
ret
;##############################################################################;
print_str_num:
push ax
call print
pop  ax
call print_hexword
mov  si,newline
call print
ret
;##############################################################################;
check_crc:
push ds
mov  ax,1000h
mov  ds,ax 
mov  cx,5h
mov  dx,9FF0h
mov  al,0
xor  bx,bx

bigloop:
push cx
mov  cx,0FFFFh

add_byte: 
add  al,[bx]
inc  bx
loop add_byte
add  al,[bx]

mov  bx,ds
add  bx,1000h
mov  ds,bx
xor  bx,bx
pop  cx
loop bigloop

mov  cx,dx
add_byte1: 
add  al,[bx]
inc  bx
loop add_byte1

pop  ds
ret      
;##############################################################################;
dsmsg     db "DS=$",0
esmsg     db "ES=$",0
ssmsg     db "SS=$",0
spmsg     db "SP=$",0
newline   db 0Dh,0Ah,0
intermsg  db "T",0

crcmsg     db "crc=",0
err_msg    db "Error:",0
disk_a_msg db "A=",0
disk_b_msg db "B=",0
disk_c_msg db "C=",0
disk_d_msg db "D=",0
pos_msg    db "Sector=",0

TrkSecs    dw 9h ;00012h  

;##############################################################################;
include "unitools.asm"
;##############################################################################;
rb ROM_SIZE*2048-3-$
dw 086AEh           ;E8086 signature
rb 1                ;Checksum
;##############################################################################;